{************************************************************************}
{************************************************************************}
{* Modul:       ScrEing.Pas                                             *}
{************************************************************************}
{* Inhalt:      Men- und Masken-Funktionen                             *}
{************************************************************************}
{* Funktion:    Ausgabe von Bildschirm-Mens und Steuerung von Eingabe- *}
{*              Masken.                                                 *}
{************************************************************************}
{* Version:     1.01                                                    *}
{* Autor:       Thomas Mainka                                           *}
{* Quelle:                                                              *} 
{* Datum:       31.Mai.1995                                             *}
{* Vernderung: Korrektur des CTRL-C Abbruchs.                          *}
{************************************************************************}
{* Revision:    1.00                                                    *}
{************************************************************************}
{* Routinen:    VRead                                                   *}
{*              VWrite                                                  *}
{*              VPos                                                    *}
{*              FSRead                                                  *}
{*              FSWrite                                                 *}
{*              FCRead                                                  *}
{*              FCWrite                                                 *}
{*              FIRead                                                  *}
{*              FRRead                                                  *}
{*              Eing                                                    *}
{*              FileExists                                              *}
{*              ESchr                                                   *}
{*              MPrt                                                    *}
{*              GetScreen                                               *}
{*              AddScr                                                  *}
{*              Displ                                                   *}
{*              GetName                                                 *}
{************************************************************************}

Unit ScrEing;
interface

uses     Crt;

Const    High      =  14;
         HiBlink   = 142;
         HiInv     =  30;
         HiInvBlink= 190;
         HiUnder   =  14;
         HiUnderBlink = 142;
         InvUnder  =  62;

Type     MaskPar   = record
                       FAnf  : Integer;
                       FLae  : Integer;
                       FNam  : String[40];
                       DTyp  : Char;
                     end;
         ScrForm   = record
                       Ch : Char;
                       At : Byte;
                     end;
         ScrType   = Array[1..2000] of ScrForm;
         SString   = String[132];
         DString   = String[20];

Var      Video     : ScrType absolute $B800:$0000;
         VText     : ScrType;
         Maske     : Array[1..25] of MaskPar;
         MFile     : File of MaskPar;
         FA,FE     : Array[1..25] of Integer;
         k         : Integer;
         ScrMem    : ScrType;
         ScrFile   : File of ScrType;


Procedure VRead;
Procedure VWrite;
Function VPos(X,Y:integer):Integer;

Function FSRead(i,j:Integer):SString;
{$V- }
Procedure FSWrite(j:Integer;S:SString);
{$V+ }

Function FCRead(j:Integer):Char;
Procedure FCWrite(j:Integer;C:Char);

Function FIRead(i,j:Integer):Integer;
Function FRRead(i,j:Integer):Real;

Procedure Eing (Var C:Char);
Function FileExists(DName:DString;PH:Boolean): Boolean;
Procedure ESchr;
Procedure MPrt(DName:DString);

Procedure GetScreen(Var Screen:ScrType;Var OK:Boolean; DName:DString);
Procedure AddScr(Var Scr1:ScrType;Scr2:ScrType);

Procedure Displ(S:SString; Hell:Byte);
Procedure GetName(Var Name:SString);

implementation

{************************************************************************}
{* Routine:     VRead                                                   *}
{************************************************************************}
{* Inhalt:      Kopieren des Bildschirmspeichers in Sicherungsfeld      *}
{*              Bildschirm                                              *}
{* Definition:  Procedure VRead                                         *}
{************************************************************************}

Procedure VRead;
begin
  VText:=Video;
end;

{************************************************************************}
{* Routine:     VWrite                                                  *}
{************************************************************************}
{* Inhalt:      Kopieren des Sicherungsfeldes in den Bildschirmspeicher *}
{* Definition:  Procedure VWrite                                        *}
{************************************************************************}

Procedure VWrite;
begin
  Video:=VText;
end;

{************************************************************************}
{* Routine:     VPos                                                    *}
{************************************************************************}
{* Inhalt:      Umwandlung einer XY-Koordinate in Speicherposition im   *}
{*              Bildschirmspeicher                                      *}
{* Definition:  Function VPos (X,Y:Integer):Integer                     *}
{* Parameter:   X            - X-Koordinate                             *}
{*              Y            - Y-Koordinate                             *}
{* Rckwert:    VPos         - Bildschimspeicher-Adresse                *}
{************************************************************************}

Function VPos(X,Y:Integer): Integer;
begin
  VPos:=(Y-1)*80+X;
end;

{************************************************************************}
{* Routine:     FSRead                                                  *}
{************************************************************************}
{* Inhalt:      Lesen eines Strings aus der Eingabe-Maske               *}
{* Definition:  Function FSRead(i,j:Integer):SString                    *}
{* Parameter:   i            - Lnge des Masken-Feldes                  *}
{*              j            - Startposition des Feldes                 *}
{* Rckwert:    FSRead       - gelesener String                         *}
{************************************************************************}

Function FSRead(i,j:Integer):SString;
Var      k         : Integer;
         S         : SString;
         P         : Boolean;
begin
   P:=True;
   S:='';
   for k:=1 to i do
     if P and ((VText[j+k-1].Ch<>'.') or (VText[j+k].Ch<>'.')) then
       S:=S+VText[j+k-1].Ch
     else P:=False;
   FSRead:=S;
end;

{************************************************************************}
{* Routine:     FSWrite                                                 *}
{************************************************************************}
{* Inhalt:      Ausgabe eines Strings in der Bildschirm-Maske           *}
{* Definition:  Procedure FSWrite (J:Integer;S:SString)                 *}
{* Parameter:   J            - Startposition des Feldes                 *}
{*              S            - Auszugebender String                     *}
{************************************************************************}

{$V- }
Procedure FSWrite(j:Integer;S:SString);
Var      k         : Integer;
begin
   for k:=1 to Length(S) do VText[j+k-1].Ch:=S[k];
end;
{$V+ }

{************************************************************************}
{* Routine:     FCRead                                                  *}
{************************************************************************}
{* Inhalt:      Lesen eines Characters aus der Eingabe-Maske            *}
{* Definition:  Function FCRead(j:Integer):Char                         *}
{* Parameter:   j            - Position des Feldes                      *}
{* Rckwert:    FRRead       - gelesene Real-Zahl                       *}
{************************************************************************}

Function FCRead(j:Integer):Char;
begin
   if VText[j].Ch<>'.' then FCRead:=VText[j].Ch
   else FCRead:=' ';
end;

{************************************************************************}
{* Routine:     FCWrite                                                 *}
{************************************************************************}
{* Inhalt:      Ausgabe eines Characters in der Bildschirm-Maske        *}
{* Definition:  Procedure FCWrite (J:Integer;C:Char)                    *}
{* Parameter:   J            - Position des Characters                  *}
{*              C            - Auszugebender Character                  *}
{************************************************************************}

Procedure FCWrite(j:Integer;C:Char);
begin
   if C<>' ' then VText[j].Ch:=C
   else VText[j].Ch:='.';
end;

{************************************************************************}
{* Routine:     FIRead                                                  *}
{************************************************************************}
{* Inhalt:      Lesen einer Integer-Zahl aus der Eingabe-Maske          *}
{* Definition:  Function FRRead(i,j:Integer):Integer                    *}
{* Parameter:   i            - Lnge des Masken-Feldes                  *}
{*              j            - Startposition des Feldes                 *}
{* Rckwert:    FIRead       - gelesene Integer-Zahl                    *}
{************************************************************************}

Function FIRead(i,j:Integer):Integer;
Var      k,l       : Integer;
         S         : SString;
         P         : Boolean;
begin
  P:=True;
  S:='';
  for k:=1 to i do
    if (VText[j+k].Ch in ['1'..'0','+','-']) and P then
      S:=S+VText[j+k].Ch
    else P:=False;
  Val(s,l,k);
  FIRead:=l;
end;

{************************************************************************}
{* Routine:     FRRead                                                  *}
{************************************************************************}
{* Inhalt:      Lesen einer Real-Zahl aus der Eingabe-Maske             *}
{* Definition:  Function FRRead(i,j:Integer):Real                       *}
{* Parameter:   i            - Lnge des Masken-Feldes                  *}
{*              j            - Startposition des Feldes                 *}
{* Rckwert:    FRRead       - gelesene Real-Zahl                       *}
{************************************************************************}

Function FRRead(i,j:Integer):Real;
Var      k         : Integer;
         S         : SString;
         P         : Boolean;
         r         : Real;
begin
  P:=True;
  S:='';
  for k:=1 to i do
    if (VText[j+k].Ch in ['1'..'0','E','.','+','-']) and P then
      S:=S+VText[j+k].Ch
    else P:=False;
  Val(s,r,k);
  FRRead:=r;
end;

{************************************************************************}
{* Routine:     Eing                                                    *}
{************************************************************************}
{* Inhalt:      Eingabe eines Buchstabens (Tastendrucks) mit UpCase-    *}
{*              Wandlung zur Men-Auswahl                               *}
{* Definition:  Procedure Eing (Var C:Char)                             *}
{* Parameter:   C            - Eingegebener Buchstabe                   *}
{************************************************************************}

Procedure Eing(Var C:Char);
begin
   repeat until KeyPressed;
   C:=ReadKey;
   C:=UpCase(C);
end;

{************************************************************************}
{* Routine:     FileExists                                                      *}
{************************************************************************}
{* Inhalt:      Laden einer Eingabe-Maske und Anzeige direkt auf den    *}
{*              Bildschirm                                              *}
{* Definition:  Procedure MPrt (DName:DString)                          *}
{* Parameter:   DName        - Datei-Name der Eingabe-Maske             *}
{************************************************************************}

Function FileExists(DName:DString;PH:Boolean): Boolean;
Var      Datei     : File;
         Fehler    : Word;
begin
   Assign(Datei,DName);
   {$I- } Reset(Datei); {$I+ }
   Fehler:=IOResult;
   FileExists:=(Fehler=0);
   if (Fehler<>0) and PH then begin
     Writeln(DName,' nicht vorhanden oder defekt!');
     Writeln('Zur Fehlerquittierung bitte eine Taste drcken.');
     repeat until Keypressed;
   end;
   if Fehler=0 then Close(Datei);
end;

{************************************************************************}
{* Routine:     ESchr                                                   *}
{************************************************************************}
{* Inhalt:      Ausfllen einer Eingabe-Maske                           *}
{* Definition:  Procedure ESchr                                         *}
{************************************************************************}

Procedure ESchr;
Var      c,d       : Char;
         i         : Integer;
begin
   i:=1;
   GotoXY(FA[i] mod 80,FA[i] div 80+1);
   repeat
     repeat until Keypressed;
     c:=ReadKey;
     if c=#0 then begin
       d:=ReadKey;
       case d of
         'K' : GotoXY(WhereX-1,WhereY);
         'M' : GotoXY(WhereX+1,WhereY);
         '' : begin
                 if i=1 then i:=k
                 else i:=i-1;
                 GotoXY(FA[i] mod 80,FA[i] div 80+1);
               end;
      end;
     end
     else if c<>#9 then Write(c)
       else begin
         if i<k then i:=i+1
         else i:=1;
         GotoXY(FA[i] mod 80,FA[i] div 80+1);
       end;
     if (WhereY-1)*80+WhereX>FE[i] then begin
       if i<k then i:=i+1
       else i:=1;
       GotoXY(FA[i] mod 80,FA[i] div 80+1);
     end;
     if (WhereY-1)*80+WhereX<FA[i] then begin
       if i=1 then i:=k
       else i:=i-1;
       GotoXY(FE[i] mod 80,FE[i] div 80+1);
     end;
   until c=#13;
end;

{************************************************************************}
{* Routine:     MPrt                                                    *}
{************************************************************************}
{* Inhalt:      Laden einer Eingabe-Maske und Anzeige direkt auf den    *}
{*              Bildschirm                                              *}
{* Definition:  Procedure MPrt (DName:DString)                          *}
{* Parameter:   DName        - Datei-Name der Eingabe-Maske             *}
{************************************************************************}

Procedure MPrt(DName:DString);
Var      d         : Char;
         i,j       : Integer;
begin
   if FileExists(DName,True) then begin
     Assign(MFile,DName);
     Reset(MFile);
     k:=0;
     while not EOF(MFile) do begin
       k:=k+1;
       Read(MFile,Maske[k]);
     end;
     Close(MFile);
     for i:=1 to 2000 do
       With VText[i] do begin
         Ch:=' ';
         At:=HiUnder;
       end;
     for i:=1 to k do
       with Maske[i] do begin
         FA[i]:=FAnf+Length(FNam);
         FE[i]:=FA[i]+FLae-1;
         for j:=1 to length(FNam) do VText[j+FAnf-1].Ch:=FNam[j];
         if DTyp='A' then d:='.' else d:='#';
         for j:=1 to FLae do VText[j+FA[i]-1].Ch:=d;
         FNam:='';
       end;
     VWrite;
   end;
end;

{************************************************************************}
{* Routine:     GetScreen                                               *}
{************************************************************************}
{* Inhalt:      Lesen eines Men-Bildschirms von einer SCR-Datei        *}
{* Definition:  Procedure GetScreen (Var Screen:ScrType;                *}
{*                                   Var OK:Boolean; DName:DString)     *}
{* Parameter:   Screen       - geladener Men-Bildschirm                *}
{*              OK           - Men-Bildschirm existiert                *}
{*              DName        - Datei-Name des Men-Bildschirms          *}
{************************************************************************}

Procedure GetScreen(Var Screen:ScrType;Var OK:Boolean; DName:DString);
begin
  OK:=FileExists(DName+'.SCR',True);
  if OK then begin
    Assign(ScrFile,DName+'.SCR');
    Reset(ScrFile);
    Read(Scrfile,Screen);
    Close(ScrFile);
  end;
end;

{************************************************************************}
{* Routine:     AddScr                                                  *}
{************************************************************************}
{* Inhalt:      Zusammenmischen zweier Menbildschirme zur direkten     *}
{*              Bildschirmausgabe                                       *}
{* Definition:  Procedure AddScr(Var Scr1:ScrType;Scr2:ScrType)         *}
{* Parameter:   Scr1         - Ziel-Menbildschirm                      *}
{*              Scr2         - Quell-Menbildschirm                     *}
{************************************************************************}

Procedure AddScr(Var Scr1:ScrType;Scr2:ScrType);
Var      i         : Integer;
begin
  for i:=1 to 2000 do
    if Scr2[i].Ch<>' ' then begin
      Scr1[i].Ch:=Scr2[i].Ch;
      Scr1[i].At:=Scr2[i].At;
    end;
end;

{************************************************************************}
{* Routine:     Displ                                                   *}
{************************************************************************}
{* Inhalt:      Ausgabe eines Strings mit Farbattribut direkt auf den   *}
{*              Bildschirm                                              *}
{* Definition:  Procedure Displ (S:SString; Hell:Byte)                  *}
{* Parameter:   S            - Auszugebener String                      *}
{*              Hell         - Farbattribut                             *}
{************************************************************************}

Procedure Displ(S:SString; Hell:Byte);
Var      i,P       : Integer;
begin
  P:=VPos(WhereX,WhereY)-1;
  For i:=1 to Length(S) do
    with Video[P+i] do begin
      Ch:=S[i];
      At:=Hell;
    end;
  GotoXY(WhereX+Length(S),WhereY);
end;

{************************************************************************}
{* Routine:     GetName                                                 *}
{************************************************************************}
{* Inhalt:      Eingabe eines Strings mit direkter Bildschirmausgabe    *}
{* Definition:  Procedure GetName (Var Name:SString)                    *}
{* Parameter:   Name         - Eingegebener String                      *}
{************************************************************************}

Procedure GetName(Var Name:SString);
Var      Taste     : Char;
begin
  Name:='';
  repeat
    repeat until KeyPressed;
    Taste:=ReadKey;
    if Taste in [' '..'}','','','','','',''] then begin
      Name:=Name+Taste;
      with Video[VPos(WhereX,WhereY)] do begin
        Ch:=Taste;
        At:=HiInv;
      end;
      GotoXY(WhereX+1,WhereY);
    end;
    if Ord(Taste)=8 then begin
      If Length(Name)>0 then begin
        GotoXY(WhereX-1,WhereY);
        Video[VPos(WhereX,WhereY)].Ch:=' ';
        Delete(Name,Length(Name),1);
      end;
    end;
  until (Ord(Taste)=13);
end;

begin
end.
